package and06.geonotes

import android.content.Context
import android.content.Intent
import android.location.Location
import android.location.LocationListener
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.location.LocationManager
import android.util.Log
import android.view.View
import android.widget.*
import android.Manifest
import android.app.ProgressDialog.show
import android.content.ActivityNotFoundException
import android.content.DialogInterface
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Environment
import java.text.DateFormat
import android.view.Menu
import android.view.MenuItem
import androidx.appcompat.app.AlertDialog
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.util.Date

class GatherActivity : AppCompatActivity() {
    var minTime = 5000L // in Millisekunden
    var minDistance = 5.0f // in Metern
    var aktuellesProjekt = Projekt(Date().getTime(), "")
    var aktuelleNotiz: Notiz? = null
    val locationListener = NoteLocationListener()

    companion object {
        val NOTIZEN = "notizen"
        val INDEX_AKTUELLE_NOTIZ = "index_aktuelle_notiz"
        val PREFERENCES = "preferences"
        val ID_ZULETZT_GEOEFFNETES_PROJEKT = "id_zuletzt_geoeffnetes_projekt"
        val AKTUELLES_PROJEKT = "aktuelles_projekt"
        val AKTUELLE_NOTIZ = "aktuelle_notiz"
        val TILE_TYPE = "tile_type"
    }

    inner class NoteLocationListener : LocationListener {
        override fun onLocationChanged(location: Location) {
            Log.d(javaClass.simpleName, "Empfangene Geodaten:\n$location")
            /*            val textview = findViewById<TextView>(R.id.textview_output)
                        textview.append("\nEmpfangene Geodaten:\n$location")*/
        }
    }

    inner class SpinnerProviderItemSelectedListener : AdapterView.OnItemSelectedListener {
        override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
            if (findViewById<ToggleButton>(R.id.togglebutton_lokalisierung).isChecked()) {
                val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
                locationManager.removeUpdates(locationListener)
                val provider = (view as TextView).text.toString()
                try {
                    locationManager.requestLocationUpdates(
                        provider,
                        minTime,
                        minDistance,
                        locationListener
                    )
                } catch (ex: SecurityException) {
                    Log.e(
                        javaClass.simpleName,
                        "Erforderliche Berechtigung $ex.toString() nicht erteilt"
                    )
                }
            }
        }

        override fun onNothingSelected(parent: AdapterView<*>?) {
        }
    }

    fun initSpinnerProviders() {
        val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
        val providers = locationManager.getProviders(true)
        Log.d(javaClass.simpleName, "Verfügbare Provider:")
        providers.forEach {
            Log.d(javaClass.simpleName, "Provider: $it")
        }
        val spinner = findViewById<Spinner>(R.id.spinner_provider)
        spinner.adapter = ArrayAdapter<String>(this, R.layout.list_item, providers)
        spinner.onItemSelectedListener = SpinnerProviderItemSelectedListener()
        if (providers.contains("gps"))
            spinner.setSelection(providers.indexOf("gps"))
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        val locationIndex = permissions.indexOf(Manifest.permission.ACCESS_FINE_LOCATION)
        if (grantResults[locationIndex] == PackageManager.PERMISSION_GRANTED) {  //deprecated source
            initSpinnerProviders()
        } else {
            Toast.makeText(
                this,
                "Standortbestimmung nicht erlaubt -- nur eingeschränkte Funktionalität verfügbar",
                Toast.LENGTH_LONG
            ).show()
        }
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContentView(R.layout.activity_gather)

        requestPermissions(
            arrayOf<String>(
                Manifest.permission.WRITE_EXTERNAL_STORAGE,
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.ACCESS_COARSE_LOCATION
            ), 0
        )

        setSupportActionBar(findViewById(R.id.toolbar)) // deprecated source

        val textview = findViewById<TextView>(R.id.textview_aktuelles_projekt)
        textview.append(aktuellesProjekt.getDescription())

        val notiz: Notiz?

        if (savedInstanceState != null) {
            if (Build.VERSION.SDK_INT >= 33) { // TIRAMISU
                aktuellesProjekt =
                    savedInstanceState.getParcelable(AKTUELLES_PROJEKT, Projekt::class.java)!!
                notiz =
                    savedInstanceState.getParcelable(AKTUELLE_NOTIZ, Notiz::class.java)
            } else {
                aktuellesProjekt =
                    savedInstanceState.getParcelable(AKTUELLES_PROJEKT)!!
                notiz =
                    savedInstanceState.getParcelable(AKTUELLE_NOTIZ)
            }
            if (notiz != null) aktuelleNotiz = notiz!!
            return
        }

        val projektId = getSharedPreferences(PREFERENCES, Context.MODE_PRIVATE).getLong(
            ID_ZULETZT_GEOEFFNETES_PROJEKT,
            0
        )
        if (projektId != 0L) {
            val database = GeoNotesDatabase.getInstance(this)
            CoroutineScope(Dispatchers.Main).launch {
                var projekt: Projekt? = null
                withContext(Dispatchers.IO) {
                    projekt = database.projekteDao().getProjekt(projektId)
                }
                if (projekt != null) {
                    with(AlertDialog.Builder(this@GatherActivity)) {
                        setMessage("Projekt \"${projekt?.getDescription()}\" weiter bearbeiten?")
                        setPositiveButton("Ja", DialogInterface.OnClickListener { dialog, id ->
                            aktuellesProjekt = projekt!!
                            val textview = findViewById<TextView>(R.id.textview_aktuelles_projekt)
                            textview.text =
                                getString(R.string.aktuelles_projekt_prefix) + aktuellesProjekt.getDescription()
                        })
                        setNegativeButton("Nein", DialogInterface.OnClickListener { dialog, id ->
                        })
                        show()
                    }
                }
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()

        val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
        locationManager.removeUpdates(locationListener)

        getSharedPreferences(PREFERENCES, Context.MODE_PRIVATE).edit()
            .putLong(ID_ZULETZT_GEOEFFNETES_PROJEKT, aktuellesProjekt.id).apply()
        Log.d(javaClass.simpleName, "ID_ZULETZT_GEOEFFNETES_PROJEKT: " + aktuellesProjekt.id)
    }

    override fun onPause() {
        super.onPause()
        val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
        locationManager.removeUpdates(locationListener)
    }

    override fun onResume() {
        super.onResume()
        if (findViewById<ToggleButton>(R.id.togglebutton_lokalisierung).isChecked()) {
            val spinner = findViewById<Spinner>(R.id.spinner_provider)
            val provider = spinner.selectedItem as String
            val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
            try {
                locationManager.requestLocationUpdates(
                    provider,
                    minTime,
                    minDistance,
                    locationListener
                )
            } catch (ex: SecurityException) {
                Log.e(
                    javaClass.simpleName,
                    "Erforderliche Berechtigung ${ex.toString()} nicht erteilt"
                )
            }
        }
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        outState.putParcelable(AKTUELLES_PROJEKT, aktuellesProjekt)
        if (aktuelleNotiz != null) outState.putParcelable(AKTUELLE_NOTIZ, aktuelleNotiz)
    }

    @Deprecated("Deprecated in Java")
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)

        if (resultCode != RESULT_OK) return
        if (requestCode != 0) return
        val extras = data?.getExtras() ?: return
        val notizId = extras.getLong(NoteMapActivity.AKTUELLE_NOTIZ_ID)
        val database = GeoNotesDatabase.getInstance(this)
        Log.d(javaClass.simpleName, "Selected Notiz-ID: $notizId")
        CoroutineScope(Dispatchers.Main).launch {
            var notiz: Notiz? = null
            withContext(Dispatchers.IO) {
                notiz = database.notizenDao().getNotiz(notizId)
            }
            if (notiz != null) {
                aktuelleNotiz = notiz
                findViewById<TextView>(R.id.edittext_thema).text = aktuelleNotiz?.thema
                findViewById<TextView>(R.id.edittext_notiz).text = aktuelleNotiz?.notiz
            }
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_gather_activity, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        val id = item.itemId
        when (id) {
            R.id.item_foot, R.id.item_bicycle, R.id.item_car, R.id.item_car_fast, R.id.item_desktop -> {
                item.setChecked(true)
                val map = hashMapOf<Int, Pair<Long, Float>>(
                    R.id.item_desktop to (60000L to 10.0f),
                    R.id.item_foot to (9000L to 10.0f),
                    R.id.item_bicycle to (4000L to 25.0f),
                    R.id.item_car to (4000L to 50.0f),
                    R.id.item_car_fast to (4000L to 100.0f)
                )
                val pair = map.get(id) ?: return true
                minTime = pair.first
                minDistance = pair.second
                Toast.makeText(
                    this,
                    "Neues GPS-Intervall \"$item\". Bitte Lokalisierung neu starten.",
                    Toast.LENGTH_LONG
                ).show()
                return true
            }

            R.id.menu_projekt_bearbeiten -> {
                openProjektBearbeitenDialog()
            }

            R.id.menu_projekt_auswaehlen -> {
                openProjektAuswaehlenDialog()
            }

            R.id.menu_notiz_loeschen -> {
                notizLoeschen()
            }

            R.id.menu_projekt_versenden -> {
                projektVersenden()
            }

            R.id.menu_osm_oeffnen -> {
                openNavigation()
            }

            R.id.menu_ghost -> {
                callGhostApi()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    fun callGhostApi() {
        CoroutineScope(Dispatchers.Main).launch {
            withContext(Dispatchers.IO) {
                ghostAPI.register()
                ghostAPI.sendPosting(title=aktuelleNotiz!!.thema, html=aktuelleNotiz!!.notiz, author="yova@freedomhost.de")
            /*createPost()*/
                /*listPosts*/
                /*createPost()*/

            }
        }
    }
    fun projektVersenden() {
        if
                (!Environment.MEDIA_MOUNTED.equals(Environment.getExternalStorageState())) {
            Toast.makeText(this, "External Storage nicht verfügbar", Toast.LENGTH_LONG).show()
            return
        }
        val database = GeoNotesDatabase.getInstance(this)
        GlobalScope.launch {
            val notizen = database.notizenDao().getNotizen(aktuellesProjekt.id)
            if (notizen.isNullOrEmpty()) {
                Toast.makeText(this@GatherActivity, "keine Notizen vorhanden", Toast.LENGTH_LONG).show()
                return@launch
            }
            val generator = GpxGenerator()
            val uri = generator.createGpxFile(this@GatherActivity, notizen, aktuellesProjekt.getDescription())
            with(Intent(Intent.ACTION_SEND)) {
                type = "text/xml"
                putExtra(Intent.EXTRA_EMAIL, arrayOf("info@joefixit.eu"))
                putExtra(Intent.EXTRA_SUBJECT, aktuellesProjekt.getDescription())
                putExtra(Intent.EXTRA_STREAM, uri)
                startActivity(Intent.createChooser(this, "Mail versenden"))
            }
        }
    }

    private fun notizLoeschen() {
        if (aktuelleNotiz == null) {
            Toast.makeText(this, "Notiz noch nicht gespeichert", Toast.LENGTH_LONG).show()
            return
        }
        val id: Long = aktuelleNotiz!!.id as Long
        val textViewThema = findViewById<TextView>(R.id.edittext_thema)
        val textViewNotiz = findViewById<TextView>(R.id.edittext_notiz)

        val database = GeoNotesDatabase.getInstance(this)
        CoroutineScope(Dispatchers.Main).launch {
            var notizen: List<Notiz>? = null
            withContext(Dispatchers.IO) {
                notizen = database.notizenDao().getNotizen(aktuellesProjekt.id)
            }

            if (!notizen.isNullOrEmpty() && notizen!!.size > 1) {
                withContext(Dispatchers.IO) {
                    database.notizenDao().deleteNotiz(id)
                    notizen = database.notizenDao().getNotizen(aktuellesProjekt.id)
                }
                Toast.makeText(this@GatherActivity, "Notiz gelöscht.", Toast.LENGTH_LONG).show()
                aktuelleNotiz = null
                textViewThema.text = ""
                textViewNotiz.text = ""

            } else {
                // Ganzes Projekt löschen
                with(AlertDialog.Builder(this@GatherActivity)) {
                    setTitle("Löschen der letzten Notiz löscht das Projekt. Fortfahren?")
                    setNegativeButton("Abbrechen") { _, _ ->
                        Log.d(javaClass.simpleName, "Abbrechen")
                        return@setNegativeButton
                    }
                    setPositiveButton("OK") { _, _ ->
                        CoroutineScope(Dispatchers.Main).launch {
                            withContext(Dispatchers.IO) {
                                database.notizenDao().deleteNotiz(id)
                                database.projekteDao().deleteProjekt(aktuellesProjekt.id)
                            }
                            Toast.makeText(this@GatherActivity, "Projekt gelöscht.", Toast.LENGTH_LONG).show()
                            textViewThema.text = ""
                            textViewNotiz.text = ""
                            aktuellesProjekt = Projekt(Date().getTime(), "")
                            findViewById<TextView>(R.id.textview_aktuelles_projekt).text = getString(R.string.aktuelles_projekt_prefix) + aktuellesProjekt.getDescription()

                        }
                    }
                    show()
                }
            }
        }
    }

    fun onToggleButtonLokalisierenClick(view: View?) {
            val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
            if ((view as ToggleButton).isChecked()) {
                val spinner = findViewById<Spinner>(R.id.spinner_provider)
                val provider = spinner.selectedItem as String?
                if (provider == null) {
                    Toast.makeText(
                        this,
                        "Erforderliche Berechtigungen wurden nicht erteilt",
                        Toast.LENGTH_LONG
                    ).show()
                    (view as ToggleButton).setChecked(false)
                    return
                }
                try {
                    locationManager.requestLocationUpdates(
                        provider,
                        minTime,
                        minDistance,
                        locationListener
                    )
                    Log.d(javaClass.simpleName, "Lokalisierung gestartet")
                } catch (ex: SecurityException) {
                    Log.e(
                        javaClass.simpleName,
                        "Erforderliche Berechtigung $ex.toString() nicht erteilt"
                    )
                }
            } else {
                locationManager.removeUpdates(locationListener)
                Log.d(javaClass.simpleName, "Lokalisierung beendet")
            }
        }


    fun onButtonNotizSpeichernClick(view: View) {
            val themaText = findViewById<TextView>(R.id.edittext_thema).text.toString().trim()
            val notizText = findViewById<TextView>(R.id.edittext_notiz).text.toString().trim()
            if (themaText.isEmpty() || notizText.isEmpty()) {
                Toast.makeText(this, "Thema und Notiz dürfen nicht leer sein", Toast.LENGTH_LONG)
                    .show()
                return
            }
            var lastLocation: Location? = null
            var provider = ""
            try {
                val locationManager = getSystemService(LOCATION_SERVICE) as LocationManager
                val spinner = findViewById<Spinner>(R.id.spinner_provider)
                provider = spinner.selectedItem as String
                lastLocation = locationManager.getLastKnownLocation(provider)
            } catch (ex: SecurityException) {
                Log.e(
                    javaClass.simpleName,
                    "Erforderliche Berechtigung $ex.toString() nicht erteilt"
                )
            }
            if (lastLocation == null) {
                Toast.makeText(
                    this,
                    "Noch keine Geoposition ermittelt. Bitte später nochmal versuchen",
                    Toast.LENGTH_LONG
                ).show()
                return
            }
// Projekt speichern
            val database = GeoNotesDatabase.getInstance(this)
            GlobalScope.launch {
                val id = database.projekteDao().insertProjekt(aktuellesProjekt)
                Log.d(
                    javaClass.simpleName,
                    "Projekt $aktuellesProjekt mit id=$id in Datenbank geschrieben"
                )

                if (aktuelleNotiz == null && lastLocation != null) {
                    // Location speichern:
                    database.locationsDao().insertLocation(
                        Location(
                            lastLocation.latitude,
                            lastLocation.longitude,
                            lastLocation.altitude,
                            provider
                        )
                    )
                    // Notiz speichern:
                    aktuelleNotiz = Notiz(
                        null,
                        aktuellesProjekt.id,
                        lastLocation.latitude,
                        lastLocation.longitude,
                        themaText,
                        notizText
                    )
                    aktuelleNotiz?.id = database.notizenDao().insertNotiz(aktuelleNotiz!!)
                    Log.d(javaClass.simpleName, "Notiz $aktuelleNotiz gespeichert")
                } else {
                    // Notiz aktualisieren
                    aktuelleNotiz?.thema = themaText
                    aktuelleNotiz?.notiz = notizText
                    database.notizenDao().insertNotiz(aktuelleNotiz!!)
                    Log.d(javaClass.simpleName, "Notiz $aktuelleNotiz aktualisiert")
                }
            }

            with(AlertDialog.Builder(this)) {
                setTitle("Notiz weiter bearbeiten?")
                setNegativeButton("Nein") { _, _ ->
                    aktuelleNotiz = null
                    findViewById<TextView>(R.id.edittext_thema).text = ""
                    findViewById<TextView>(R.id.edittext_notiz).text = ""
                }
                setPositiveButton("Ja") { _, _ -> }
                show()
            }
        }

    fun openProjektBearbeitenDialog() {
            val dialogView = layoutInflater.inflate(R.layout.dialog_projekt_bearbeiten, null)
            val textView =
                dialogView.findViewById<TextView>(R.id.textview_dialog_projekt_bearbeiten)
            val dateFormat = DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.SHORT)
            textView.text = dateFormat.format(Date(aktuellesProjekt.id))
            val editText =
                dialogView.findViewById<TextView>(R.id.edittext_dialog_projekt_bearbeiten)
            editText.text = aktuellesProjekt.beschreibung
            val database = GeoNotesDatabase.getInstance(this)
            with(AlertDialog.Builder(this)) {
                setView(dialogView)
                setTitle("Projektbeschreibung eingeben/ändern")
                setPositiveButton("Übernehmen", DialogInterface.OnClickListener { dialog, id ->
// Projektbeschreibung ändern
                    aktuellesProjekt.beschreibung = editText.text.toString().trim()
                    findViewById<TextView>(R.id.textview_aktuelles_projekt).text =
                        getString(R.string.aktuelles_projekt_prefix) + aktuellesProjekt.getDescription()
// Projekt-Update in Datenbank
                    GlobalScope.launch {
                        database.projekteDao().updateProjekt(aktuellesProjekt)
                        Log.d(javaClass.simpleName, "Update Projekt $aktuellesProjekt")
                        val projekte = database.projekteDao().getProjekte()
                        projekte.forEach {
                            Log.d(javaClass.simpleName, "Projekt $it")
                        }
                    }

                })
                setNegativeButton("Abbrechen", DialogInterface.OnClickListener { dialog, id -> })
                show()
            }
        }

    fun openProjektAuswaehlenDialog() {
            var notizen: List<Notiz>
            val database = GeoNotesDatabase.getInstance(this)
            CoroutineScope(Dispatchers.Main).launch() {
                var projekte: List<Projekt>? = null
                withContext(Dispatchers.IO) {
                    projekte = database.projekteDao().getProjekte()
                }
                if (projekte.isNullOrEmpty()) {
                    Toast.makeText(
                        this@GatherActivity,
                        "Noch keine Projekte vorhanden",
                        Toast.LENGTH_LONG
                    ).show()
                    return@launch
                }
                val items = ArrayList<CharSequence>()
                projekte?.forEach {
                    items.add(it.getDescription())
                }
                with(AlertDialog.Builder(this@GatherActivity)) {
                    setTitle("Projekt auswählen")
                    setItems(
                        items.toArray(emptyArray())
                    ) { _, id ->
// aktuelles Projekt auf ausgewähltes Projekt setzen:
                        aktuellesProjekt = projekte?.get(id)!!
                        val textView = findViewById<TextView>(R.id.textview_aktuelles_projekt)
                        textView.text =
                            getString(R.string.aktuelles_projekt_prefix) + aktuellesProjekt.getDescription()
// Notiz zum Projekt anzeigen:
                        CoroutineScope(Dispatchers.Main).launch {
                            withContext(Dispatchers.IO) {
                                notizen = database.notizenDao().getNotizen(aktuellesProjekt.id)
                            }
                            if (!notizen.isNullOrEmpty()) aktuelleNotiz = notizen.last()
                            else {
                                Toast.makeText(this@GatherActivity, "Projekt ist leer. Wird gelöscht. Aufräum.", Toast.LENGTH_LONG).show()
                                withContext(Dispatchers.IO) {
                                    database.projekteDao().deleteProjekt(aktuellesProjekt.id)
                                }
                            }
                            findViewById<TextView>(R.id.edittext_thema).text = aktuelleNotiz?.thema
                            findViewById<TextView>(R.id.edittext_notiz).text = aktuelleNotiz?.notiz
                        }
                    }
                    show()
                }
            }
        }

    fun onButtonVorherigeNotizClick(view: View) {
            val textViewThema = findViewById<TextView>(R.id.edittext_thema)
            val textViewNotiz = findViewById<TextView>(R.id.edittext_notiz)
            val textViewProjekt = findViewById<TextView>(R.id.textview_aktuelles_projekt)
            if (aktuelleNotiz == null || (!textViewThema.text.trim().toString()
                    .equals(aktuelleNotiz?.thema) || !textViewNotiz.text.trim().toString()
                    .equals(aktuelleNotiz?.notiz))
            ) {
                if (textViewThema.text.isNotEmpty() || textViewNotiz.text.isNotEmpty()) {
                    Toast.makeText(this, "Notiz wurde noch nicht gespeichert", Toast.LENGTH_LONG)
                        .show() // Fall 1
                    return
                }
            }

            val database = GeoNotesDatabase.getInstance(this)
            CoroutineScope(Dispatchers.Main).launch {
                var notizen: List<Notiz>? = null
                withContext(Dispatchers.IO) {
                    if (aktuelleNotiz == null) { // Fall 2
                        notizen = database.notizenDao().getNotizen(aktuellesProjekt.id)
                    } else { // Fall 3
                        notizen = database.notizenDao()
                            .getPreviousNotizen(aktuelleNotiz?.id!!, aktuellesProjekt.id)
                    }
                }
                if (!notizen.isNullOrEmpty()) {
                    aktuelleNotiz = notizen?.last()
                    textViewThema.text = aktuelleNotiz?.thema
                    textViewNotiz.text = aktuelleNotiz?.notiz
                }
            }
        }

    fun onButtonNaechsteNotizClick(view: View) {
            val textViewThema = findViewById<TextView>(R.id.edittext_thema)
            val textViewNotiz = findViewById<TextView>(R.id.edittext_notiz)
            if (aktuelleNotiz == null) {
                if (textViewThema.text.isNotEmpty() || textViewNotiz.text.isNotEmpty()) {
                    Toast.makeText(this, "Notiz wurde noch nicht gespeichert", Toast.LENGTH_LONG)
                        .show() // Fall 1
                }
                return // Fall 1 und Fall 2
            }
            val database = GeoNotesDatabase.getInstance(this)
            CoroutineScope(Dispatchers.Main).launch {
                var notizen: List<Notiz>? = null
                withContext(Dispatchers.IO) {
                    notizen =
                        database.notizenDao()
                            .getNextNotizen(aktuelleNotiz?.id!!, aktuellesProjekt.id)
                }
                if (!notizen.isNullOrEmpty()) { // Fall 3a)
                    aktuelleNotiz = notizen?.first()
                    textViewThema.text = aktuelleNotiz?.thema
                    textViewNotiz.text = aktuelleNotiz?.notiz
                } else { // Fall 3b)
                    aktuelleNotiz = null
                    textViewThema.text = ""
                    textViewNotiz.text = ""
                    withContext(Dispatchers.IO) {
                        notizen = database.notizenDao().getNotizen(aktuellesProjekt.id)
                    }
                }
            }
        }

    fun onButtonStandortAnzeigenClick(view: View) {
        if (aktuelleNotiz == null) {
            Toast.makeText(this, "Bitte Notiz auswählen oder speichern", Toast.LENGTH_LONG)
                .show()
            return
        }
        val database = GeoNotesDatabase.getInstance(this)
        CoroutineScope(Dispatchers.Main).launch {
            var notizen: List<Notiz>? = null
            withContext(Dispatchers.IO) {
                notizen = database.notizenDao().getNotizen(aktuellesProjekt.id)
            }
            if (!notizen.isNullOrEmpty()) {
                val intent = Intent(this@GatherActivity, NoteMapActivity::class.java)
                intent.putParcelableArrayListExtra(NOTIZEN, ArrayList<Notiz>(notizen!!))
                intent.putExtra(INDEX_AKTUELLE_NOTIZ, notizen?.indexOf(aktuelleNotiz!!))
                with(AlertDialog.Builder(this@GatherActivity)) {
                    setSingleChoiceItems(R.array.map_types,-1, DialogInterface.OnClickListener { dialog, id ->
                        Log.i(javaClass.simpleName, "Ausgewählter map typ: " + id)
                        if (id ==0 ) {
                            intent.setClass(this@GatherActivity, OsmWebViewActivity::class.java)
                        } else {
                            intent.setClass(this@GatherActivity, NoteMapActivity::class.java)
                        }
                        if (id == 2) {
                            intent.putExtra(TILE_TYPE, 1)
                        }
                    })

                    setPositiveButton("OK", DialogInterface.OnClickListener { dialog, id ->
                        startActivityForResult(intent, 0)
                    })
                    setNegativeButton("Abbrechen", DialogInterface.OnClickListener { dialog, id -> })
                    show()
                }
            }
        }
    }


    fun openNavigation() {
        if (aktuelleNotiz == null) {
            Toast.makeText(this, "Bitte Notiz auswählen oder speichern", Toast.LENGTH_LONG)
                .show()
            return
        }

        val geouri = "geo:" + aktuelleNotiz!!.latitude.toString() +','+ aktuelleNotiz!!.longitude.toString()
        Log.i(javaClass.simpleName, "geouri: ${geouri}")
        val naviIntent = Intent(Intent.ACTION_VIEW).setData(Uri.parse(geouri))

        try {
            startActivity(naviIntent)
        } catch (e: ActivityNotFoundException) {
            Toast.makeText(this, "Keine geeignete Anwendung installiert", Toast.LENGTH_LONG).show()
            Log.i(javaClass.simpleName, e.toString())
        }
    }
}
