package and06.geonotes

import android.location.Location
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.widget.ImageButton
import androidx.activity.OnBackPressedCallback
import androidx.appcompat.app.AppCompatActivity
import org.osmdroid.config.Configuration
import org.osmdroid.tileprovider.tilesource.OnlineTileSourceBase
import org.osmdroid.tileprovider.tilesource.TileSourceFactory
import org.osmdroid.tileprovider.tilesource.XYTileSource
import org.osmdroid.util.GeoPoint
import org.osmdroid.util.MapTileIndex
import org.osmdroid.views.MapView
import org.osmdroid.views.overlay.Marker
import java.io.File


class NoteMapActivity : AppCompatActivity() {
    companion object {
        val AKTUELLE_NOTIZ_ID = "aktuelle_notiz_id"
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        val GoogleSat: OnlineTileSourceBase = object : XYTileSource(
            "Google-Sat",
            0, 19, 256, ".png", arrayOf(
                "https://mt0.google.com",
                "https://mt1.google.com",
                "https://mt2.google.com",
                "https://mt3.google.com"
            )
        ) {
            override fun getTileURLString(pTileIndex: Long): String {
                return baseUrl + "/vt/lyrs=s&x=" + MapTileIndex.getX(pTileIndex) + "&y=" + MapTileIndex.getY(
                    pTileIndex
                ) + "&z=" + MapTileIndex.getZoom(pTileIndex)
            }
        }

        TileSourceFactory.addTileSource(GoogleSat)
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_notemap)
        val osmConfig = Configuration.getInstance()
        osmConfig.userAgentValue = packageName
        val basePath = File(cacheDir.absolutePath, "osmdroid")
        osmConfig.osmdroidBasePath = basePath
        val tileCache = File(osmConfig.osmdroidBasePath, "tile")
        osmConfig.osmdroidTileCache = tileCache
        val map = findViewById<MapView>(R.id.mapview)


        val extras = intent.extras ?: return
        val notizen : ArrayList<Notiz>

        if (Build.VERSION.SDK_INT >= 33) { // TIRAMISU
            notizen = intent.getParcelableArrayListExtra(GatherActivity.NOTIZEN, Notiz::class.java) ?: return
        } else {
            notizen = intent.getParcelableArrayListExtra<Notiz>(GatherActivity.NOTIZEN) ?: return
        }

        val ourTileType = extras.getInt(GatherActivity.TILE_TYPE)
        if (ourTileType != 0) {
            map.setTileSource(GoogleSat)
        } else {
            map.setTileSource(TileSourceFactory.OpenTopo)
        }

        notizen.forEach {
            val marker = Marker(map)
            marker.position = GeoPoint(it.latitude, it.longitude)
            marker.title = it.thema
            marker.snippet = it.notiz
            map.overlays.add(marker)
        }
        val controller = map.controller
        var indexAktuelleNotiz = extras.getInt(GatherActivity.INDEX_AKTUELLE_NOTIZ)
        val aktuelleNotiz = notizen.get(indexAktuelleNotiz)

        controller.setCenter(GeoPoint(aktuelleNotiz.latitude, aktuelleNotiz.longitude))
        controller.setZoom(10.0)

        val buttonPrevious = findViewById<ImageButton>(R.id.button_previous_notiz)
        buttonPrevious.setOnClickListener{
            indexAktuelleNotiz = if (indexAktuelleNotiz == 0) notizen.size-1 else indexAktuelleNotiz-1
            val notiz = notizen.get(indexAktuelleNotiz)
            controller.setCenter(GeoPoint(notiz.latitude, notiz.longitude))
        }

        val buttonNext = findViewById<ImageButton>(R.id.button_next_notiz)
        buttonNext.setOnClickListener {
            indexAktuelleNotiz = (indexAktuelleNotiz + 1) % notizen.size
            val notiz = notizen.get(indexAktuelleNotiz)
            controller.setCenter(GeoPoint(notiz.latitude, notiz.longitude))
        }
        onBackPressedDispatcher.addCallback(this /* lifecycle owner */, object : OnBackPressedCallback(true) {
            override fun handleOnBackPressed() {
            // Back is pressed... Finishing the activity
                val pushIntent = getIntent()
                pushIntent.putExtra(AKTUELLE_NOTIZ_ID, notizen.get(indexAktuelleNotiz).id)
                setResult(RESULT_OK, pushIntent)
                finish()
            }
        })
    }

    fun decimalToSexagesimal(latitude: Double, longitude: Double): String {
        val latDegrees = latitude.toInt()
        val lonDegrees = longitude.toInt()
        val latTempMinutes = Math.abs((latitude - latDegrees) * 60)
        val lonTempMinutes = Math.abs((longitude - lonDegrees) * 60)

        val latMinutes = latTempMinutes.toInt()
        val lonMinutes = lonTempMinutes.toInt()
        val latTempSeconds = (latTempMinutes - latMinutes) * 60
        val lonTempSeconds = (lonTempMinutes - lonMinutes) * 60
// auf drei Stellen runden
        val latSeconds = Math.round(latTempSeconds * 1000)/1000.0
        val lonSeconds = Math.round(lonTempSeconds * 1000)/1000.0

        val latDegreesString = Math.abs(latDegrees).toString() + if (latitude < 0) "°S " else "°N "
        val lonDegreesString = Math.abs(lonDegrees).toString() + if (longitude < 0) "°W " else "°O "
        return lonDegreesString + lonMinutes + "\' " + lonSeconds + "\'\' / " + latDegreesString + latMinutes + "\' " + latSeconds + "\'\'"
    }

    override fun onResume() {
        super.onResume()
        val map = findViewById<MapView>(R.id.mapview)
        map.onResume()
    }

    override fun onPause() {
        super.onPause()
        val map = findViewById<MapView>(R.id.mapview)
        map.onPause()
    }
}